@@ -64,6 +64,8 @@ pub fn add_subcommand(command: Command) -> Command {
 }
 /// Check the [`ArgMatches`] for the subcommand added by [`add_subcommand`].
 ///
+/// All printing is done to `stderr` to avoid clogging up potential output.
+///
 /// # Returns
 ///
 /// Ignore if this returns [`None`].
@@ -77,10 +79,12 @@ pub fn test_subcommand(matches: &ArgMatches, mut command: Command) -> Option<Res
                 .map(Into::into)
                 .ok_or(())
                 .or_else(|()| {
-                    println!("Getting your shell.");
-                    get_shell::get_shell_name().map_err(|_| {
-                        "failed to detect shell, please explicitly supply it".to_owned()
-                    })
+                    eprintln!("Getting your shell.");
+                    query_shell::get_shell()
+                        .map_err(|_| {
+                            "failed to detect shell, please explicitly supply it".to_owned()
+                        })
+                        .map(|shell| shell.to_str().to_owned())
                 })?;
             name.make_ascii_lowercase();
             match name.as_str() {
@@ -103,8 +107,12 @@ pub fn test_subcommand(matches: &ArgMatches, mut command: Command) -> Option<Res
         } else {
             let mut buffer = Vec::with_capacity(512);
             clap_complete::generate(shell, &mut command, &bin_name, &mut buffer);
-            write_shell(shell, &buffer, &bin_name)
-                .map_err(|err| format!("insufficient privileges: {}", err))?;
+            write_shell(shell, &buffer, &bin_name).map_err(|err| match err.kind() {
+                io::ErrorKind::PermissionDenied => {
+                    "Failed to write shell. Permission denied.".to_owned()
+                }
+                _ => format!("Failed to write shell: {}", err),
+            })?;
             Ok(())
         }
     })
@@ -123,7 +131,7 @@ fn write_shell(shell: Shell, data: &[u8], bin_name: &str) -> Result<(), io::Erro
     if let Some(parent) = path.parent() {
         create_dir_all(parent)?;
     }
-    println!("Writing completions to {}", path.display());
+    eprintln!("Writing completions to {}", path.display());
     fs::write(path, data)?;
     Ok(())
 }
